"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveIndividual = void 0;
const camelcase_1 = __importDefault(require("camelcase"));
const utils_1 = require("./utils");
function resolveIndividualImpl({ idlAccounts, programId, provider, args, accounts, path = [], resolver, idlIx, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const newPath = [...path, (0, camelcase_1.default)(idlAccounts.name)];
        try {
            if (idlAccounts.accounts) {
                let resolved = 0;
                const subAccounts = idlAccounts.accounts;
                for (let k = 0; k < subAccounts.length; k += 1) {
                    const subAccount = subAccounts[k];
                    resolved += yield resolveIndividualImpl({
                        idlAccounts: subAccount,
                        programId,
                        provider,
                        args,
                        accounts,
                        path: newPath,
                        resolver,
                        idlIx,
                    });
                }
                return resolved;
            }
            else {
                let resolved = 0;
                let value = (0, utils_1.get)(accounts, newPath);
                if (!value) {
                    value = yield resolver({
                        programId,
                        provider,
                        path: newPath,
                        args,
                        accounts,
                        idlIx,
                    });
                    if (value) {
                        resolved = 1;
                    }
                }
                (0, utils_1.set)(accounts, newPath, value);
                return resolved;
            }
        }
        catch (e) {
            console.error(`Error while resolving ${newPath}`, e);
            throw e;
        }
    });
}
/**
 * Allows custom account resolution by functionaly operating on one account at a time.
 *
 * Check the `path` arg to see the account name being operated on, and use `accounts` , `args` , and `provider` to fill in any
 * details necessary
 *
 * @param resolver
 * @returns
 */
function resolveIndividual(resolver) {
    return ({ idlIx, args, accounts, programId, provider }) => __awaiter(this, void 0, void 0, function* () {
        let resolved = 0;
        for (let k = 0; k < idlIx.accounts.length; k += 1) {
            resolved += yield resolveIndividualImpl({
                idlAccounts: idlIx.accounts[k],
                programId,
                provider,
                resolver,
                args,
                accounts,
                idlIx,
            });
        }
        return {
            accounts,
            resolved
        };
    });
}
exports.resolveIndividual = resolveIndividual;
//# sourceMappingURL=individual.js.map