"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.searchAssets = exports.getAssetsByOwner = exports.getAssetProof = exports.getAsset = void 0;
const web3_js_1 = require("@solana/web3.js");
const axios_1 = __importDefault(require("axios"));
// @ts-ignore
const bs58_1 = __importDefault(require("bs58"));
function getAsset(url, assetId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield axios_1.default.post(url, {
                jsonrpc: "2.0",
                method: "getAsset",
                id: "rpd-op-123",
                params: { id: assetId.toBase58() },
                headers: {
                    "Cache-Control": "no-cache",
                    Pragma: "no-cache",
                    Expires: "0",
                },
            });
            const result = response.data.result;
            if (result) {
                return toAsset(result);
            }
        }
        catch (error) {
            console.error(error);
            throw error;
        }
    });
}
exports.getAsset = getAsset;
function toAsset(result) {
    return Object.assign(Object.assign({}, result), { id: new web3_js_1.PublicKey(result.id), grouping: result.grouping && new web3_js_1.PublicKey(result.grouping), compression: Object.assign(Object.assign({}, result.compression), { leafId: result.compression.leaf_id, dataHash: result.compression.data_hash &&
                Buffer.from(bs58_1.default.decode(result.compression.data_hash)), creatorHash: result.compression.creator_hash &&
                Buffer.from(bs58_1.default.decode(result.compression.creator_hash)), assetHash: result.compression.asset_hash &&
                Buffer.from(bs58_1.default.decode(result.compression.asset_hash)), tree: result.compression.tree && new web3_js_1.PublicKey(result.compression.tree) }), ownership: Object.assign(Object.assign({}, result.ownership), { delegate: result.ownership.delegate && new web3_js_1.PublicKey(result.ownership.delegate), owner: result.ownership.owner && new web3_js_1.PublicKey(result.ownership.owner) }) });
}
function getAssetProof(url, assetId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield axios_1.default.post(url, {
                jsonrpc: "2.0",
                method: "getAssetProof",
                id: "rpd-op-123",
                params: { id: assetId.toBase58() },
                headers: {
                    "Cache-Control": "no-cache",
                    Pragma: "no-cache",
                    Expires: "0",
                },
            });
            const result = response.data.result;
            if (result) {
                return {
                    root: new web3_js_1.PublicKey(result.root),
                    proof: result.proof.map((p) => new web3_js_1.PublicKey(p)),
                    nodeIndex: result.node_index,
                    leaf: new web3_js_1.PublicKey(result.leaf),
                    treeId: new web3_js_1.PublicKey(result.tree_id),
                };
            }
        }
        catch (error) {
            console.error(error);
            throw error;
        }
    });
}
exports.getAssetProof = getAssetProof;
function getAssetsByOwner(url, wallet, { sortBy = { sortBy: "created", sortDirection: "asc" }, limit = 50, page = 1, before = "", after = "", } = {}) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield axios_1.default.post(url, {
                jsonrpc: "2.0",
                method: "getAssetsByOwner",
                id: "rpd-op-123",
                params: [wallet, sortBy, limit, page, before, after],
                headers: {
                    "Cache-Control": "no-cache",
                    Pragma: "no-cache",
                    Expires: "0",
                },
            });
            return (_a = response.data.result) === null || _a === void 0 ? void 0 : _a.items.map(toAsset);
        }
        catch (error) {
            console.error(error);
            throw error;
        }
    });
}
exports.getAssetsByOwner = getAssetsByOwner;
function searchAssets(url, _a) {
    var _b;
    var { creatorVerified = true, sortBy = { sortBy: "created", sortDirection: "asc" }, page = 1 } = _a, rest = __rest(_a, ["creatorVerified", "sortBy", "page"]);
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const response = yield axios_1.default.post(url, {
                jsonrpc: "2.0",
                method: "searchAssets",
                id: "get-assets-op-1",
                params: Object.assign({ page,
                    creatorVerified,
                    sortBy }, rest),
                headers: {
                    "Cache-Control": "no-cache",
                    Pragma: "no-cache",
                    Expires: "0",
                },
            });
            return (_b = response.data.result) === null || _b === void 0 ? void 0 : _b.items.map(toAsset);
        }
        catch (error) {
            console.error(error);
            throw error;
        }
    });
}
exports.searchAssets = searchAssets;
//# sourceMappingURL=mplAssetAPI.js.map