"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withPriorityFees = exports.estimatePrioritizationFee = void 0;
const web3_js_1 = require("@solana/web3.js");
const MAX_RECENT_PRIORITY_FEE_ACCOUNTS = 128;
// Borrowed with love from https://github.com/blockworks-foundation/mango-v4/blob/57a9835aa8f636b6d231ba2c4008bfe89cbf08ba/ts/client/src/client.ts#L4552
/**
 * Returns an estimate of a prioritization fee for a set of instructions.
 *
 * The estimate is based on the median fees of writable accounts that will be involved in the transaction.
 *
 * @param ixs - the instructions that make up the transaction
 * @returns prioritizationFeeEstimate -- in microLamports
 */
function estimatePrioritizationFee(connection, ixs, basePriorityFee) {
    return __awaiter(this, void 0, void 0, function* () {
        const writableAccounts = ixs
            .map((x) => x.keys.filter((a) => a.isWritable).map((k) => k.pubkey))
            .flat();
        const uniqueWritableAccounts = [
            ...new Set(writableAccounts.map((x) => x.toBase58())),
        ]
            .map((a) => new web3_js_1.PublicKey(a))
            .slice(0, MAX_RECENT_PRIORITY_FEE_ACCOUNTS);
        const priorityFees = yield connection.getRecentPrioritizationFees({
            lockedWritableAccounts: uniqueWritableAccounts,
        });
        if (priorityFees.length < 1) {
            return Math.max(basePriorityFee || 0, 1);
        }
        // get max priority fee per slot (and sort by slot from old to new)
        const groupedBySlot = priorityFees.reduce((acc, fee) => {
            const key = fee.slot;
            if (!acc[key]) {
                acc[key] = [];
            }
            acc[key].push(fee);
            return acc;
        }, {});
        const maxFeeBySlot = Object.keys(groupedBySlot).reduce((acc, slot) => {
            acc[slot] = groupedBySlot[slot].reduce((max, fee) => {
                return fee.prioritizationFee > max.prioritizationFee ? fee : max;
            });
            return acc;
        }, {});
        const maximumFees = Object.values(maxFeeBySlot).sort((a, b) => a.slot - b.slot);
        // get median of last 20 fees
        const recentFees = maximumFees.slice(Math.max(maximumFees.length - 20, 0));
        const mid = Math.floor(recentFees.length / 2);
        const medianFee = recentFees.length % 2 !== 0
            ? recentFees[mid].prioritizationFee
            : (recentFees[mid - 1].prioritizationFee +
                recentFees[mid].prioritizationFee) /
                2;
        return Math.max(basePriorityFee || 1, Math.ceil(medianFee));
    });
}
exports.estimatePrioritizationFee = estimatePrioritizationFee;
function withPriorityFees({ connection, computeUnits, instructions, basePriorityFee }) {
    return __awaiter(this, void 0, void 0, function* () {
        const estimate = yield estimatePrioritizationFee(connection, instructions, basePriorityFee);
        const cu = computeUnits ? [web3_js_1.ComputeBudgetProgram.setComputeUnitLimit({
                units: computeUnits,
            })] : [];
        return [
            ...cu,
            web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: estimate,
            }),
            ...instructions,
        ];
    });
}
exports.withPriorityFees = withPriorityFees;
//# sourceMappingURL=priorityFees.js.map