"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.proofArgsAndAccounts = void 0;
const mplAssetAPI_1 = require("./mplAssetAPI");
const spl_account_compression_1 = require("@solana/spl-account-compression");
function proofArgsAndAccounts({ connection, assetId, assetEndpoint, getAssetFn = mplAssetAPI_1.getAsset, getAssetProofFn = mplAssetAPI_1.getAssetProof, }) {
    return __awaiter(this, void 0, void 0, function* () {
        // @ts-ignore
        const endpoint = assetEndpoint || connection._rpcEndpoint;
        const asset = yield getAssetFn(endpoint, assetId);
        if (!asset) {
            throw new Error("No asset with ID " + assetId.toBase58());
        }
        const assetProof = yield getAssetProofFn(endpoint, assetId);
        if (!assetProof) {
            throw new Error("No asset proof with ID " + assetId.toBase58());
        }
        const { compression: { leafId }, } = asset;
        const { root, proof, leaf, treeId } = assetProof;
        const canopy = yield (yield spl_account_compression_1.ConcurrentMerkleTreeAccount.fromAccountAddress(connection, treeId)).getCanopyDepth();
        return {
            asset,
            args: {
                dataHash: asset.compression.dataHash.toJSON().data,
                creatorHash: asset.compression.creatorHash.toJSON().data,
                root: root.toBuffer().toJSON().data,
                index: leafId,
            },
            accounts: {
                merkleTree: treeId,
            },
            remainingAccounts: proof.slice(0, proof.length - canopy).map((p) => {
                return {
                    pubkey: p,
                    isWritable: false,
                    isSigner: false,
                };
            }),
        };
    });
}
exports.proofArgsAndAccounts = proofArgsAndAccounts;
//# sourceMappingURL=proofArgsAndAccounts.js.map