"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBalanceDeltas = exports.isAta = exports.createMint = exports.createMintInstructions = exports.createAtaAndMint = exports.createAta = exports.createAtaAndMintInstructions = exports.createAtaAndTransfer = exports.createAtaAndTransferInstructions = exports.mintTo = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
function mintTo(provider, mint, amount, destination) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintTx = new web3_js_1.Transaction();
        mintTx.add((0, spl_token_1.createMintToInstruction)(mint, destination, provider.wallet.publicKey, amount));
        try {
            yield provider.sendAndConfirm(mintTx);
        }
        catch (e) {
            console.log('Error', e, e.logs);
            if (e.logs) {
                console.error(e.logs.join('\n'));
            }
            throw e;
        }
    });
}
exports.mintTo = mintTo;
function createAtaAndTransferInstructions(provider, mint, amount, from = provider.wallet.publicKey, to = provider.wallet.publicKey, payer = provider.wallet.publicKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const toAta = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, to, true);
        const instructions = [];
        if (!(yield provider.connection.getAccountInfo(toAta))) {
            instructions.push((0, spl_token_1.createAssociatedTokenAccountInstruction)(payer, toAta, to, mint));
        }
        const fromAta = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, from, true);
        if (amount != 0) {
            instructions.push((0, spl_token_1.createTransferInstruction)(fromAta, toAta, from, BigInt(amount.toString())));
        }
        return {
            instructions,
            toAta,
        };
    });
}
exports.createAtaAndTransferInstructions = createAtaAndTransferInstructions;
function createAtaAndTransfer(provider, mint, amount, from = provider.wallet.publicKey, to = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey, confirmOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const transferIx = new web3_js_1.Transaction();
        const { instructions, toAta } = yield createAtaAndTransferInstructions(provider, mint, amount, from, to, payer);
        if (instructions.length > 0)
            transferIx.add(...instructions);
        try {
            if (instructions.length > 0)
                yield provider.sendAndConfirm(transferIx, undefined, confirmOptions);
        }
        catch (e) {
            console.log('Error', e, e.logs);
            if (e.logs) {
                console.error(e.logs.join('\n'));
            }
            throw e;
        }
        return toAta;
    });
}
exports.createAtaAndTransfer = createAtaAndTransfer;
function createAtaAndMintInstructions(provider, mint, amount, to = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const ata = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, to, true);
        const instructions = [];
        if (!(yield provider.connection.getAccountInfo(ata))) {
            instructions.push((0, spl_token_1.createAssociatedTokenAccountInstruction)(payer, ata, to, mint));
        }
        if (amount != 0) {
            instructions.push((0, spl_token_1.createMintToInstruction)(mint, ata, authority, BigInt(amount.toString())));
        }
        return {
            instructions,
            ata,
        };
    });
}
exports.createAtaAndMintInstructions = createAtaAndMintInstructions;
function createAta(provider, mint, owner = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey, confirmOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const tx = new web3_js_1.Transaction();
        const ata = yield (0, spl_token_1.getAssociatedTokenAddress)(mint, owner, true);
        if (!(yield provider.connection.getAccountInfo(ata))) {
            tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(payer, ata, owner, mint));
        }
        try {
            yield provider.sendAndConfirm(tx, undefined, confirmOptions);
        }
        catch (e) {
            console.log('Error', e, e.logs);
            if (e.logs) {
                console.error(e.logs.join('\n'));
            }
            throw e;
        }
        return ata;
    });
}
exports.createAta = createAta;
function createAtaAndMint(provider, mint, amount, to = provider.wallet.publicKey, authority = provider.wallet.publicKey, payer = provider.wallet.publicKey, signers = [], confirmOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintTx = new web3_js_1.Transaction();
        const { instructions, ata } = yield createAtaAndMintInstructions(provider, mint, amount, to, authority, payer);
        if (instructions.length > 0)
            mintTx.add(...instructions);
        try {
            if (instructions.length > 0)
                yield provider.sendAndConfirm(mintTx, signers.length == 0 ? undefined : signers, confirmOptions);
        }
        catch (e) {
            console.log('Error', e, e.logs);
            if (e.logs) {
                console.error(e.logs.join('\n'));
            }
            throw e;
        }
        return ata;
    });
}
exports.createAtaAndMint = createAtaAndMint;
function createMintInstructions(provider, decimals, mintAuthority, freezeAuthority = null, mintKeypair = web3_js_1.Keypair.generate()) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintKey = mintKeypair.publicKey;
        return [
            web3_js_1.SystemProgram.createAccount({
                fromPubkey: provider.wallet.publicKey,
                newAccountPubkey: mintKey,
                space: 82,
                lamports: yield provider.connection.getMinimumBalanceForRentExemption(82),
                programId: spl_token_1.TOKEN_PROGRAM_ID,
            }),
            yield (0, spl_token_1.createInitializeMintInstruction)(mintKeypair.publicKey, decimals, mintAuthority, freezeAuthority),
        ];
    });
}
exports.createMintInstructions = createMintInstructions;
function createMint(provider, decimals, mintAuthority, freezeAuthority = null, mintKeypair = web3_js_1.Keypair.generate()) {
    return __awaiter(this, void 0, void 0, function* () {
        const tx = new web3_js_1.Transaction();
        tx.add(...(yield createMintInstructions(provider, decimals, mintAuthority, freezeAuthority, mintKeypair)));
        try {
            yield provider.sendAndConfirm(tx, [mintKeypair]);
        }
        catch (e) {
            console.log('Error', e, e.logs);
            if (e.logs) {
                console.error(e.logs.join('\n'));
            }
            throw e;
        }
        return mintKeypair.publicKey;
    });
}
exports.createMint = createMint;
const isAta = (connection, address, mint) => __awaiter(void 0, void 0, void 0, function* () {
    let tokenAccount;
    try {
        tokenAccount = yield (0, spl_token_1.getAccount)(connection, address);
    }
    catch (e) {
        return;
    }
    if (mint && !tokenAccount.mint.equals(mint)) {
        return;
    }
    const expectedAta = (0, spl_token_1.getAssociatedTokenAddressSync)(tokenAccount.mint, tokenAccount.owner, true);
    if (expectedAta.equals(address)) {
        return { owner: tokenAccount.owner };
    }
    return;
});
exports.isAta = isAta;
const getBalanceDeltas = (conn, sig, mint = new web3_js_1.PublicKey('rndrizKT3MK1iimdxRdWabcF7Zg7AR5T4nud4EkHBof')) => __awaiter(void 0, void 0, void 0, function* () {
    const tx = yield conn.getParsedTransaction(sig, {
        maxSupportedTransactionVersion: 0,
        commitment: 'confirmed',
    });
    if (!tx) {
        throw new Error(`tx ${tx} not found`);
    }
    const bals = {};
    for (const b of tx.meta.postTokenBalances) {
        if (b.mint == mint.toString()) {
            bals[b.owner] = Number(b.uiTokenAmount.amount);
        }
    }
    let sender = '';
    for (const b of tx.meta.preTokenBalances) {
        if (b.mint == mint.toString()) {
            bals[b.owner] -= Number(b.uiTokenAmount.amount);
            if (bals[b.owner] < 0) {
                sender = b.owner;
            }
        }
    }
    return {
        sender: new web3_js_1.PublicKey(sender),
        deltas: bals,
        executedAt: new Date(tx.blockTime * 1000),
    };
});
exports.getBalanceDeltas = getBalanceDeltas;
/*export async function createNft(
  provider: anchor.AnchorProvider,
  recipient: PublicKey,
  data: any = {},
  collectionKey?: PublicKey,
  mintKeypair: Keypair = Keypair.generate(),
  holderKey: PublicKey = provider.wallet.publicKey
): Promise<{ mintKey: PublicKey; collectionKey: PublicKey | undefined }> {
  const mintKey = mintKeypair.publicKey;

  const instructions = await createMintInstructions(
    provider,
    0,
    provider.wallet.publicKey,
    provider.wallet.publicKey,
    mintKeypair
  );
  const [metadata] = PublicKey.findProgramAddressSync(
    [
      Buffer.from("metadata", "utf8"),
      METADATA_PROGRAM_ID.toBuffer(),
      mintKey.toBuffer(),
    ],
    METADATA_PROGRAM_ID
  );
  instructions.push(
    await createCreateMetadataAccountV3Instruction(
      {
        metadata,
        mint: mintKey,
        mintAuthority: provider.wallet.publicKey,
        payer: provider.wallet.publicKey,
        updateAuthority: provider.wallet.publicKey,
      },
      {
        createMetadataAccountArgsV3: {
          data: {
            name: "test",
            symbol: "TST",
            uri: "https://shdw-drive.genesysgo.net/6tcnBSybPG7piEDShBcrVtYJDPSvGrDbVvXmXKpzBvWP/dc.json",
            sellerFeeBasisPoints: 10,
            creators: [
              {
                address: holderKey,
                verified: true,
                share: 100,
              },
            ],
            collection: collectionKey
              ? { key: collectionKey, verified: false }
              : null,
            uses: null,
            ...data,
          },
          isMutable: true,
          collectionDetails: null,
        },
      }
    )
  );

  const { instructions: mintInstrs } = await createAtaAndMintInstructions(
    provider,
    mintKeypair.publicKey,
    1,
    recipient
  );
  instructions.push(...mintInstrs);

  if (collectionKey) {
    const [collectionMetadataAccount] = PublicKey.findProgramAddressSync(
      [
        Buffer.from("metadata", "utf8"),
        METADATA_PROGRAM_ID.toBuffer(),
        collectionKey.toBuffer(),
      ],
      METADATA_PROGRAM_ID
    );
    const [collectionMasterEdition] = PublicKey.findProgramAddressSync(
      [
        Buffer.from("metadata", "utf8"),
        METADATA_PROGRAM_ID.toBuffer(),
        collectionKey.toBuffer(),
        Buffer.from("edition", "utf8"),
      ],
      METADATA_PROGRAM_ID
    );
    const instruction = createVerifyCollectionInstruction({
      metadata: metadata,
      collectionAuthority: provider.wallet.publicKey,
      payer: provider.wallet.publicKey,
      collectionMint: collectionKey,
      collection: collectionMetadataAccount,
      collectionMasterEditionAccount: collectionMasterEdition,
    });
    instructions.push(instruction);
  }

  const [edition] = PublicKey.findProgramAddressSync(
    [
      Buffer.from("metadata", "utf8"),
      METADATA_PROGRAM_ID.toBuffer(),
      mintKey.toBuffer(),
      Buffer.from("edition", "utf8"),
    ],
    METADATA_PROGRAM_ID
  );
  instructions.push(
    createCreateMasterEditionV3Instruction(
      {
        edition,
        mint: mintKey,
        updateAuthority: provider.wallet.publicKey,
        mintAuthority: provider.wallet.publicKey,
        payer: provider.wallet.publicKey,
        metadata,
      },
      {
        createMasterEditionArgs: {
          maxSupply: 0,
        },
      }
    )
  );

  const tx = new Transaction();
  tx.add(...instructions);

  try {
    await provider.sendAndConfirm(tx, [mintKeypair]);
  } catch (e: any) {
    console.log("Error", e, e.logs);
    if (e.logs) {
      console.error(e.logs.join("\n"));
    }
    throw e;
  }

  return {
    mintKey,
    collectionKey,
  };
}*/
//# sourceMappingURL=token.js.map