import { PublicKey } from "@solana/web3.js";
import axios from "axios";
// @ts-ignore
import base58 from "bs58";
export async function getAsset(url, assetId) {
    try {
        const response = await axios.post(url, {
            jsonrpc: "2.0",
            method: "getAsset",
            id: "rpd-op-123",
            params: { id: assetId.toBase58() },
            headers: {
                "Cache-Control": "no-cache",
                Pragma: "no-cache",
                Expires: "0",
            },
        });
        const result = response.data.result;
        if (result) {
            return toAsset(result);
        }
    }
    catch (error) {
        console.error(error);
        throw error;
    }
}
function toAsset(result) {
    return {
        ...result,
        id: new PublicKey(result.id),
        grouping: result.grouping && new PublicKey(result.grouping),
        compression: {
            ...result.compression,
            leafId: result.compression.leaf_id,
            dataHash: result.compression.data_hash &&
                Buffer.from(base58.decode(result.compression.data_hash)),
            creatorHash: result.compression.creator_hash &&
                Buffer.from(base58.decode(result.compression.creator_hash)),
            assetHash: result.compression.asset_hash &&
                Buffer.from(base58.decode(result.compression.asset_hash)),
            tree: result.compression.tree && new PublicKey(result.compression.tree),
        },
        ownership: {
            ...result.ownership,
            delegate: result.ownership.delegate && new PublicKey(result.ownership.delegate),
            owner: result.ownership.owner && new PublicKey(result.ownership.owner),
        },
    };
}
export async function getAssetProof(url, assetId) {
    try {
        const response = await axios.post(url, {
            jsonrpc: "2.0",
            method: "getAssetProof",
            id: "rpd-op-123",
            params: { id: assetId.toBase58() },
            headers: {
                "Cache-Control": "no-cache",
                Pragma: "no-cache",
                Expires: "0",
            },
        });
        const result = response.data.result;
        if (result) {
            return {
                root: new PublicKey(result.root),
                proof: result.proof.map((p) => new PublicKey(p)),
                nodeIndex: result.node_index,
                leaf: new PublicKey(result.leaf),
                treeId: new PublicKey(result.tree_id),
            };
        }
    }
    catch (error) {
        console.error(error);
        throw error;
    }
}
export async function getAssetsByOwner(url, wallet, { sortBy = { sortBy: "created", sortDirection: "asc" }, limit = 50, page = 1, before = "", after = "", } = {}) {
    try {
        const response = await axios.post(url, {
            jsonrpc: "2.0",
            method: "getAssetsByOwner",
            id: "rpd-op-123",
            params: [wallet, sortBy, limit, page, before, after],
            headers: {
                "Cache-Control": "no-cache",
                Pragma: "no-cache",
                Expires: "0",
            },
        });
        return response.data.result?.items.map(toAsset);
    }
    catch (error) {
        console.error(error);
        throw error;
    }
}
export async function searchAssets(url, { creatorVerified = true, sortBy = { sortBy: "created", sortDirection: "asc" }, page = 1, ...rest }) {
    try {
        const response = await axios.post(url, {
            jsonrpc: "2.0",
            method: "searchAssets",
            id: "get-assets-op-1",
            params: {
                page,
                creatorVerified,
                sortBy,
                ...rest,
            },
            headers: {
                "Cache-Control": "no-cache",
                Pragma: "no-cache",
                Expires: "0",
            },
        });
        return response.data.result?.items.map(toAsset);
    }
    catch (error) {
        console.error(error);
        throw error;
    }
}
//# sourceMappingURL=mplAssetAPI.js.map