/// <reference types="node" />
import { Creator, Uses } from "@metaplex-foundation/mpl-bubblegum";
import { PublicKey } from "@solana/web3.js";
export type AssetProof = {
    root: PublicKey;
    proof: PublicKey[];
    nodeIndex: number;
    leaf: PublicKey;
    treeId: PublicKey;
};
export type Asset = {
    id: PublicKey;
    content: any;
    compression: {
        eligible: boolean;
        compressed: boolean;
        dataHash?: Buffer;
        creatorHash?: Buffer;
        assetHash?: Buffer;
        tree?: PublicKey;
        leafId?: number;
    };
    ownership: {
        owner: PublicKey;
        delegate: PublicKey;
    };
    royalty: {
        basis_points: number;
        primary_sale_happened: boolean;
    };
    mutable: boolean;
    supply: {
        edition_nonce: number | null;
    };
    grouping?: PublicKey;
    uses?: Uses;
    creators: Creator[];
};
export declare function getAsset(url: string, assetId: PublicKey): Promise<Asset | undefined>;
export declare function getAssetProof(url: string, assetId: PublicKey): Promise<AssetProof | undefined>;
export type AssetsByOwnerOpts = {
    sortBy?: {
        sortBy: "created";
        sortDirection: "asc" | "desc";
    };
    limit?: number;
    page?: number;
    before?: string;
    after?: string;
};
export declare function getAssetsByOwner(url: string, wallet: string, { sortBy, limit, page, before, after, }?: AssetsByOwnerOpts): Promise<Asset[]>;
export type SearchAssetsOpts = {
    sortBy?: {
        sortBy: "created";
        sortDirection: "asc" | "desc";
    };
    page?: number;
    collection?: string;
    ownerAddress: string;
    creatorAddress: string;
    creatorVerified?: boolean;
} & {
    [key: string]: unknown;
};
export declare function searchAssets(url: string, { creatorVerified, sortBy, page, ...rest }: SearchAssetsOpts): Promise<Asset[]>;
//# sourceMappingURL=mplAssetAPI.d.ts.map